@@ -3,19 +3,12 @@
 //!
 //! # Examples
 //!
-//! Used as version-ful:
-//!
-//! ```
-//! use vsdb::versioned::mapx_raw::MapxRawVs;
-//!
-//! // TODO
-//! let _l = MapxRawVs::new();
-//! ```
+//! Used as version-ful: [**moduler level documents**](super)
 //!
 //! Used as version-less(not recommand, use `MapxRaw` instead):
 //!
 //! ```
-//! use vsdb::{VersionName, versioned::mapx_raw::MapxRawVs};
+//! use vsdb::{VersionName, versioned::mapx_raw::MapxRawVs, VsMgmt};
 //!
 //! let mut l = MapxRawVs::new();
 //! l.version_create(VersionName(b"test")).unwrap();
@@ -41,9 +34,12 @@ mod backend;
 #[cfg(test)]
 mod test;
 
-use crate::common::{
-    BranchName, ParentBranchName, RawKey, RawValue, VersionName, INITIAL_BRANCH_NAME,
-    NULL,
+use crate::{
+    common::{
+        BranchName, ParentBranchName, RawKey, RawValue, VersionName,
+        INITIAL_BRANCH_NAME, NULL,
+    },
+    VsMgmt,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
@@ -74,14 +70,14 @@ impl MapxRawVs {
 
     /// Insert a KV to the head version of the default branch.
     #[inline(always)]
-    pub fn insert(&mut self, key: &[u8], value: &[u8]) -> Result<Option<RawValue>> {
+    pub fn insert(&self, key: &[u8], value: &[u8]) -> Result<Option<RawValue>> {
         self.inner.insert(key, value).c(d!())
     }
 
     /// Insert a KV to the head version of a specified branch.
     #[inline(always)]
     pub fn insert_by_branch(
-        &mut self,
+        &self,
         key: &[u8],
         value: &[u8],
         branch_name: BranchName,
@@ -92,14 +88,14 @@ impl MapxRawVs {
 
     /// Remove a KV from the head version of the default branch.
     #[inline(always)]
-    pub fn remove(&mut self, key: &[u8]) -> Result<Option<RawValue>> {
+    pub fn remove(&self, key: &[u8]) -> Result<Option<RawValue>> {
         self.inner.remove(key).c(d!())
     }
 
     /// Remove a KV from the head version of a specified branch.
     #[inline(always)]
     pub fn remove_by_branch(
-        &mut self,
+        &self,
         key: &[u8],
         branch_name: BranchName,
     ) -> Result<Option<RawValue>> {
@@ -398,18 +394,20 @@ impl MapxRawVs {
     pub fn clear(&mut self) {
         self.inner.clear();
     }
+}
 
+impl VsMgmt for MapxRawVs {
     /// Create a new version on the default branch.
     #[inline(always)]
-    pub fn version_create(&mut self, version_name: VersionName) -> Result<()> {
+    fn version_create(&self, version_name: VersionName) -> Result<()> {
         self.inner.version_create(version_name.0).c(d!())
     }
 
     /// Create a new version on a specified branch,
     /// NOTE: the branch must has been created.
     #[inline(always)]
-    pub fn version_create_by_branch(
-        &mut self,
+    fn version_create_by_branch(
+        &self,
         version_name: VersionName,
         branch_name: BranchName,
     ) -> Result<()> {
@@ -425,7 +423,7 @@ impl MapxRawVs {
 
     /// Check if a verison exists on default branch.
     #[inline(always)]
-    pub fn version_exists(&self, version_name: VersionName) -> bool {
+    fn version_exists(&self, version_name: VersionName) -> bool {
         self.inner
             .get_version_id(BranchName(INITIAL_BRANCH_NAME), version_name)
             .map(|id| self.inner.version_exists(id))
@@ -434,7 +432,7 @@ impl MapxRawVs {
 
     /// Check if a version exists on a specified branch(include its parents).
     #[inline(always)]
-    pub fn version_exists_on_branch(
+    fn version_exists_on_branch(
         &self,
         version_name: VersionName,
         branch_name: BranchName,
@@ -451,13 +449,13 @@ impl MapxRawVs {
 
     /// Check if a version is directly created on the default branch.
     #[inline(always)]
-    pub fn version_created(&self, version_name: VersionName) -> bool {
+    fn version_created(&self, version_name: VersionName) -> bool {
         self.version_created_on_branch(version_name, BranchName(INITIAL_BRANCH_NAME))
     }
 
     /// Check if a version is directly created on a specified branch(exclude its parents).
     #[inline(always)]
-    pub fn version_created_on_branch(
+    fn version_created_on_branch(
         &self,
         version_name: VersionName,
         branch_name: BranchName,
@@ -480,7 +478,7 @@ impl MapxRawVs {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn version_pop(&mut self) -> Result<()> {
+    fn version_pop(&self) -> Result<()> {
         self.inner.version_pop().c(d!())
     }
 
@@ -492,7 +490,7 @@ impl MapxRawVs {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn version_pop_by_branch(&mut self, branch_name: BranchName) -> Result<()> {
+    fn version_pop_by_branch(&self, branch_name: BranchName) -> Result<()> {
         self.inner
             .get_branch_id(branch_name)
             .c(d!("branch not found"))
@@ -501,14 +499,14 @@ impl MapxRawVs {
 
     /// Create a new branch based on the head of the default branch.
     #[inline(always)]
-    pub fn branch_create(&mut self, branch_name: BranchName) -> Result<()> {
+    fn branch_create(&self, branch_name: BranchName) -> Result<()> {
         self.inner.branch_create(branch_name.0).c(d!())
     }
 
     /// Create a new branch based on the head of a specified branch.
     #[inline(always)]
-    pub fn branch_create_by_base_branch(
-        &mut self,
+    fn branch_create_by_base_branch(
+        &self,
         branch_name: BranchName,
         base_branch_name: ParentBranchName,
     ) -> Result<()> {
@@ -524,8 +522,8 @@ impl MapxRawVs {
 
     /// Create a new branch based on a specified version of a specified branch.
     #[inline(always)]
-    pub fn branch_create_by_base_branch_version(
-        &mut self,
+    fn branch_create_by_base_branch_version(
+        &self,
         branch_name: BranchName,
         base_branch_name: ParentBranchName,
         base_version_name: VersionName,
@@ -545,7 +543,7 @@ impl MapxRawVs {
 
     /// Check if a branch exists or not.
     #[inline(always)]
-    pub fn branch_exists(&self, branch_name: BranchName) -> bool {
+    fn branch_exists(&self, branch_name: BranchName) -> bool {
         self.inner
             .get_branch_id(branch_name)
             .map(|id| self.inner.branch_exists(id))
@@ -560,7 +558,7 @@ impl MapxRawVs {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_remove(&mut self, branch_name: BranchName) -> Result<()> {
+    fn branch_remove(&self, branch_name: BranchName) -> Result<()> {
         if let Some(branch_id) = self.inner.get_branch_id(branch_name) {
             self.inner.branch_remove(branch_id).c(d!())
         } else {
@@ -576,7 +574,7 @@ impl MapxRawVs {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_truncate(&mut self, branch_name: BranchName) -> Result<()> {
+    fn branch_truncate(&self, branch_name: BranchName) -> Result<()> {
         self.inner
             .get_branch_id(branch_name)
             .c(d!("branch not found"))
@@ -591,8 +589,8 @@ impl MapxRawVs {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_truncate_to(
-        &mut self,
+    fn branch_truncate_to(
+        &self,
         branch_name: BranchName,
         last_version_name: VersionName,
     ) -> Result<()> {
@@ -617,7 +615,7 @@ impl MapxRawVs {
     /// while operations on branches and versions are limited to their own perspective,
     /// and should not do any tracing.
     #[inline(always)]
-    pub fn branch_pop_version(&mut self, branch_name: BranchName) -> Result<()> {
+    fn branch_pop_version(&self, branch_name: BranchName) -> Result<()> {
         self.inner
             .get_branch_id(branch_name)
             .c(d!("branch not found"))
@@ -626,7 +624,7 @@ impl MapxRawVs {
 
     /// Merge a branch to its parent branch.
     #[inline(always)]
-    pub fn branch_merge_to_parent(&mut self, branch_name: BranchName) -> Result<()> {
+    fn branch_merge_to_parent(&self, branch_name: BranchName) -> Result<()> {
         self.inner
             .get_branch_id(branch_name)
             .c(d!("branch not found"))
@@ -635,7 +633,7 @@ impl MapxRawVs {
 
     /// Check if a branch has children branches.
     #[inline(always)]
-    pub fn branch_has_children(&self, branch_name: BranchName) -> bool {
+    fn branch_has_children(&self, branch_name: BranchName) -> bool {
         self.inner
             .get_branch_id(branch_name)
             .map(|id| self.inner.branch_has_children(id))
@@ -645,7 +643,7 @@ impl MapxRawVs {
     /// Make a branch to be default,
     /// all default operations will be applied to it.
     #[inline(always)]
-    pub fn branch_set_default(&mut self, branch_name: BranchName) -> Result<()> {
+    fn branch_set_default(&mut self, branch_name: BranchName) -> Result<()> {
         self.inner
             .get_branch_id(branch_name)
             .c(d!("branch not found"))
@@ -654,14 +652,14 @@ impl MapxRawVs {
 
     /// Clean outdated versions out of the default reserved number.
     #[inline(always)]
-    pub fn prune(&mut self, reserved_ver_num: Option<usize>) -> Result<()> {
+    fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()> {
         self.inner.prune(reserved_ver_num).c(d!())
     }
 
     /// Clean outdated versions out of a specified reserved number.
     #[inline(always)]
-    pub fn prune_by_branch(
-        &mut self,
+    fn prune_by_branch(
+        &self,
         branch_name: BranchName,
         reserved_ver_num: Option<usize>,
     ) -> Result<()> {
